@@ -198,6 +198,7 @@ def openapi_to_codegen_metadata(spec: oas.OpenAPI) -> SpecMeta:
             supported_methods=iter_supported_methods(
                 common_types=common_schema_types_,
                 common_params=spec.components.parameters,
+                common_responses=spec.components.responses,
                 path=item
             )
         )
@@ -690,7 +691,8 @@ def assign_param_to_container(
 
 def iter_supported_methods(
     common_types: ResolvedTypesMap,
-    common_params: Mapping[str, oas.OperationParameter],
+    common_params: Mapping[oas.ParamTypeName, oas.OperationParameter],
+    common_responses: Mapping[oas.ResponseTypeName, oas.Response],
     path: oas.PathItem,
 ) -> SupportedMethods:
     """
@@ -713,7 +715,7 @@ def iter_supported_methods(
             if isinstance(param, oas.OperationParameter):
                 containers = assign_param_to_container(containers, param)
             elif isinstance(param, oas.Reference):
-                containers = assign_param_to_container(containers, common_params[param.ref.name])
+                containers = assign_param_to_container(containers, common_params[oas.ParamTypeName(param.ref.name)])
             else:
                 raise NotImplementedError(f'Parameter as {type(param)}')
 
@@ -763,47 +765,18 @@ def iter_supported_methods(
             except KeyError:
                 supported_status, response = list(method.responses.items())[0]
 
-            if not response.content:
-                response_types = pvector([DEFAULT_RESPONSE_TYPE._replace(
-                    name='None',
-                    common_reference_as=required_response_name
-                )])
+            if isinstance(response, oas.Response):
+                response_ = response
+            elif isinstance(response, oas.Reference):
+                try:
+                    response_ = common_responses[oas.ResponseTypeName(response.ref.name)]
+                except KeyError:
+                    raise TypeError(f'Response reference "{response.ref.name}" is not found in common types')
             else:
+                raise NotImplementedError(f'Unrecognised response type: {type(response)}')
 
-                for content_type, meta in response.content.items():
-                    if content_type.format in (oas.ContentTypeFormat.JSON,
-                                               oas.ContentTypeFormat.ANYTHING,
-                                               oas.ContentTypeFormat.FORM_URLENCODED,
-                                               oas.ContentTypeFormat.EVENT_STREAM):
-                        response_schema: Optional[oas.SchemaType] = meta.schema
-                        response_is_stream = content_type.format in (oas.ContentTypeFormat.EVENT_STREAM, oas.ContentTypeFormat.BINARY_STREAM)
-                        break
-                else:
-                    last_response = list(response.content.items())[-1][1]
-                    response_schema = last_response.schema
-
-                if response_schema is None:
-                    response_types = pvector([DEFAULT_RESPONSE_TYPE._replace(
-                        name='Any',
-                        common_reference_as=required_response_name
-                    )])
-                else:
-                    actual_response_type_name, default, response_types = recursive_resolve_schema(
-                        registry={},
-                        suggested_type_name=required_response_name,
-                        schema=response_schema,
-                        attr_name_normalizer=underscore,
-                        common_types=common_types,
-                    )
-                    if actual_response_type_name != required_response_name:
-                        response_types = response_types.append(
-                            DEFAULT_RESPONSE_TYPE._replace(
-                                name=actual_response_type_name,
-                                attrs=pvector(),
-                                common_reference_as=required_response_name
-                            )
-                        )
-
+            response_is_stream, response_types = _process_response_type(common_types, required_response_name,
+                                                                        response_, response_is_stream)
         else:
             response_types = pvector([DEFAULT_RESPONSE_TYPE])
 
@@ -830,6 +803,54 @@ def iter_supported_methods(
         )
 
 
+def _process_response_type(
+    common_types: PMap[str, TypeContext],
+    required_response_name: str,
+    response: oas.Response,
+    response_is_stream: bool
+) -> Tuple[bool,  PVector[TypeContext]]:
+    if not response.content:
+        response_types = pvector([DEFAULT_RESPONSE_TYPE._replace(
+            name='None',
+            common_reference_as=required_response_name
+        )])
+    else:
+        for content_type, meta in response.content.items():
+            if content_type.format in (oas.ContentTypeFormat.JSON,
+                                       oas.ContentTypeFormat.ANYTHING,
+                                       oas.ContentTypeFormat.FORM_URLENCODED,
+                                       oas.ContentTypeFormat.EVENT_STREAM):
+                response_schema: Optional[oas.SchemaType] = meta.schema
+                response_is_stream = content_type.format in (oas.ContentTypeFormat.EVENT_STREAM, oas.ContentTypeFormat.BINARY_STREAM)
+                break
+        else:
+            last_response = list(response.content.items())[-1][1]
+            response_schema = last_response.schema
+
+        if response_schema is None:
+            response_types = pvector([DEFAULT_RESPONSE_TYPE._replace(
+                name='Any',
+                common_reference_as=required_response_name
+            )])
+        else:
+            actual_response_type_name, default, response_types = recursive_resolve_schema(
+                registry={},
+                suggested_type_name=required_response_name,
+                schema=response_schema,
+                attr_name_normalizer=underscore,
+                common_types=common_types,
+            )
+            if actual_response_type_name != required_response_name:
+                response_types = response_types.append(
+                    DEFAULT_RESPONSE_TYPE._replace(
+                        name=actual_response_type_name,
+                        attrs=pvector(),
+                        common_reference_as=required_response_name
+                    )
+                )
+    return response_is_stream, response_types
+
+
 def infer_params_type(params: Sequence[oas.OperationParameter],
                       name_normalizer: Callable[[str], str] = lambda x: x,
                       default: TypeContext = DEFAULT_PATH_PARAMS_TYPE) -> Tuple[TypeContext, PVector[TypeContext]]:
